<?php // $Id: class_wgSmarty.php,v 1.5 2005/07/23 23:02:04 djnz Exp $

// Check for existing Smarty class to avoid collisions with other modules
// (notably Gallery). Hopefully the Smarty versions will be compatible.
if ( !class_exists('Smarty') ) {
  require_once(realpath('themes/engines/wgsmarty/smarty/libs/Smarty.class.php'));
}

/**
 * Callback function to provide a wgSmarty template's source to Smarty.
 *
 * @param $tpl_name
 *   The template to use as module:filename:theme:language.
 * @param $tpl_source
 *   Passed by ref. to return source.
 * @param &$smarty
 *   The Smarty object.
 * @return
 *  TRUE if template was found.
 */
function wg_source($tpl_name, &$tpl_source, &$wgSmarty) {
  if ( $filename = $wgSmarty->tplPath($tpl_name) ) {
    $tpl_source = file_get_contents($filename);
    return TRUE;
  } else {
    return FALSE;
  }
}


/**
 * Callback function to provide the modification time for a wgSmarty template
 * to Smarty.
 *
 * @param $tpl_name
 *   The template to use as module:filename:theme:language.
 * @param $tpl_timestamp
 *   Passed by ref. to return timestamp.
 * @param &$wgSmarty
 *   The Smarty object - note this must be of the extended wgSmarty class.
 * @return
 *  TRUE if template was found.
 */
function wg_timestamp($tpl_name, &$tpl_timestamp, &$wgSmarty) {
  if ( $filename = $wgSmarty->tplPath($tpl_name) ) {
    $tpl_timestamp = filemtime($filename);
    return TRUE;
  } else {
    return FALSE;
  }
}


function wg_secure($tpl_name, &$wgSmarty) {
    // assume all templates are secure
    return true;
}

function wg_trusted($tpl_name, &$wgSmarty) {
    // not used for templates
}


/**
 * Implements helper functions for integrating Smarty into Drupal
 */
class wgSmarty extends Smarty {

var $vars = array();

/**
 * wgSmarty class constructor.
 *
 * Initialise the wgSmarty object, including set up for the smarty parent class.
 */
  function wgSmarty() {

  	$this->vars['root'] = dirname(dirname(dirname(dirname(__FILE__)))).'/';
    $this->vars['language'] = ''; // for later development
    $this->vars['templatePath'] = 'themes/engines/wgSmarty/templates'; // path to default templates
    // $this->vars['modules'] = array(); // array of module template paths (not used - take the risk of something odd happening)

    parent::Smarty();

    // configure smarty
    $this->plugins_dir[] = $this->vars['root'] . 'themes/engines/wgsmarty/plugins';
    $this->template_dir = $this->vars['root'];
    $this->compile_dir = variable_get('wgsmarty_compile_dir', $this->vars['root'] . 'themes/engines/wgsmarty/cache_compiled');
    $this->compile_check = (bool)variable_get('wgsmarty_compile_check', false);

    // register custom template resource used by $wgSmarty->tplFetch
    $this->register_resource('wg', array('wg_source', 'wg_timestamp', 'wg_secure', 'wg_trusted'));

  } // end function wgSmarty constructor


/**
 * Display the output of a wgSmarty template.
 *
 * Simply passes arguments to tplFetch but displays the template instead of
 * returning it (may be quicker as less output buffering).
 * @return NULL
 */
function tplDisplay($template, $module='', $theme='', $language='') {
  return $this->tplFetch($template, $module, $theme, $language, TRUE);
}

/**
 * Fetch the output of a wgSmarty template.
 *
 * Uses the Smarty fetch method to fetch the output of a template. Smarty will
 * detect if the template needs (re)parsing by calling back the resource
 * functions (smarty_resource_wg_*).
 *
 * @param $module
 *   The module which owns the template (avoids collisions - use 'core' for core).
 * @param $template
 *   The filename of the template.
 * @param $theme
 *   The theme (defaults to current theme).
 * @param $language
 *   The language (defaults to current language) - not yet implemented.
 * @param $display
 *   Used by tplDisplay to remove a layer of output buffering
 * @return
 *  The HTML generated by the template.
 */
function tplFetch($template, $module='', $theme='', $language='', $display=FALSE) {

  // REVISIT could reduce the number of compiled templates by first checking if
  // a theme template exists, and setting $theme to '' if not, but will it be slower?
  if ( empty($theme) ) {
    if ( empty($this->vars['themeName']) ) {
	  // work-around late identification of current theme: this should really
	  // be done in initialisation
      $this->vars['themeName'] = substr(strrchr(path_to_theme(), '/'), 1);
	}
	$theme = $this->vars['themeName'];
  }
  return $this->fetch("wg:$module:$template:$theme:$language", NULL, NULL, $display);

} // end function tplFetch


/**
 * Get the path for a template name.
 *
 * Uses the Smarty fetch method to fetch the output of a template. Smarty will
 * detect if the template needs (re)parsing by calling back the resource
 * functions (smarty_resource_wg_*).
 *
 * @param $tplName
 *   The name of the template in the form module:template:theme:language
 * @return
 *  The filename corresponding to the template, or FALSE if not found
 */
function tplPath($tplName) {
  $tpl = explode(':', $tplName);

  // get current theme
  if ( empty($tpl[2]) ) {
    $tpl[2] = 'themes/' . $this->vars['themeName'];
  } else {
    $tpl[2] = 'themes/' . $tpl[2];
   }

  // get current language
  if ( empty($tpl[3]) ) {
    $tpl[3] = $this->vars['language'];
  }

  // turn module:template into a path
  if ( empty($tpl[0]) ) {
    $tplPath = $tpl[1];
  } else {
    $tplPath = $tpl[0].'/'.$tpl[1];
  }

  // check for theme template
  $filename = $this->vars['root'] . $tpl[2] . '/templates/' . $tplPath;
  if ( file_exists($filename) ) {
    return $filename;
  }

  // .. or if it is a module, module template
  if ( !empty($tpl[0]) ) {
    $filename = $this->vars['root'] . 'modules/' . $tpl[0] . '/templates/' . $tpl[1];
    if ( file_exists($filename) ) {
      return $filename;
    }
  }

  // .. or wgSmarty default template
  $filename = $this->vars['root'].'themes/engines/wgsmarty/templates/'.$tplPath;
  if ( file_exists($filename) ) {
    return $filename;
  }

  // there isn't a template to be found!
  return false;

} // end function tplPath

} // end class wgSmarty
